import { Request, Response } from 'express';
import dotenv from 'dotenv';
dotenv.config();
import Post, { IPost } from '../models/posts.model';
import ICoords from '../interfaces/ICoords';
import User from '../models/user.model';

async function addPost(req: Request, res: Response): Promise<Response<any, Record<string, any>> | undefined> {
    try {
        const { reiseid, userid, temperatureInCelsius, location, caption, dateTime } = req.body;

        if (!reiseid || !userid || !temperatureInCelsius || !location || !caption) {
            return res.status(400).json({ errmsg: "Malformed body, missing required fields." });
        }

        const imagePaths: string[] = [];
        if (req.files && Array.isArray(req.files)) {
            req.files.forEach((file: Express.Multer.File) => {
                const imageUrl = file.path.split("travelpicapi")[1];
                imagePaths.push("https://api.travelpic.app" + imageUrl);
            });
        }

        let postCoords: ICoords;
        try {
            postCoords = JSON.parse(location);
        } catch (error) {
            return res.status(400).json({ errmsg: "Invalid coordinates format" });
        }

        const newPost: IPost = new Post({
            reiseid,
            userid,
            imagePaths,
            temperatureInCelsius: parseFloat(temperatureInCelsius),
            postedAt: new Date(),
            coords: postCoords,
            caption,
            likedBy: [],
            dateTime: dateTime,
        });

        console.log("Neuer Post", newPost);
        const savedPost = await newPost.save();

        return res.status(200).json(savedPost);
    } catch (error) {
        return res.status(500).json({ message: error instanceof Error ? error.message : 'An error occurred' });
    }
}



async function remPost(req: Request, res: Response){

    const postIdToDelete: string = req.params.postid;

    if(!postIdToDelete){
        return res.status(500).json({errormsg: "No postid specified"});
    }


    const delpOst = await Post.findByIdAndDelete(postIdToDelete);
    return res.status(200).json(delpOst);
}

async function getAllPosts(req: Request, res: Response){

    const userid: any = req.params.userid;


    let localFollowing = [userid];
    if (!userid || userid.length == 0) {
        return res.status(500).json({error: "penner"});
    }

    const getFollowingForUser = await User.find({kc_id: userid}, 'following');

    localFollowing = localFollowing.concat(getFollowingForUser[0].following);

    if (!getFollowingForUser || getFollowingForUser.length == 0) {
        return res.status(404).json({error: "No following users"});
    }
    const getAllPostsForAllFollowers = await Post.find({userid: {$in: localFollowing}});
    getAllPostsForAllFollowers.sort((a, b) => b.postedAt.getTime() - a.postedAt.getTime());

    return res.status(200).json(getAllPostsForAllFollowers);
}

async function likePost(req: Request, res: Response){
    try {
        const {postId, likedBy} = req.body;

        const updatedPost = await Post.findByIdAndUpdate(postId, {$addToSet: {"likedBy": likedBy}}); //unique push auf den array

        return res.status(200).json(updatedPost);

    } catch (error) {
        console.log(error);
        return res.status(500).json({error: "Error while parsing Body."});
    }
}

async function dislikePost(req: Request, res: Response){
    try {
        const {postId, dislikedBy} = req.body;


        const updatedPost = await Post.findByIdAndUpdate(postId, {$pull: {"likedBy": dislikedBy}}); //unique push auf den array

        return res.status(200).json(updatedPost);

    } catch (error) {
        console.log(error);
        return res.status(500).json({error: "Error while parsing Body."});
    }
}
async function getPostsByTrip(req: Request, res: Response){
    try {
        const tripId = req.params.tripId;

        const posts = await Post.find({reiseid: tripId});
        return res.status(200).json(posts);
    } catch (error) {
        console.log(error);
        return res.status(500).json({error: "Error while parsing Body."});
    }
}

export {
    addPost,
    remPost,
    getAllPosts,
    getPostsByTrip,
    likePost,
    dislikePost
};