import { Request, Response } from 'express';
import User from '../models/user.model';
import { IUser } from '../models/user.model';
import dotenv from 'dotenv';
import * as fs from "node:fs";
import {jwtDecode} from "jwt-decode";
dotenv.config();

export const createUser = async (req: Request, res: Response): Promise<void> => {
    try {
        const { name, surname, socialHandle, email, travelId, followers, following, bio } = req.body;

        if (!name || !surname || !socialHandle || !email) {
            res.status(400).json({ message: 'Missing required fields' });
            return;
        }

        const profilePicture = req.file
            ? `${process.env.APIURL}/uploads/profilePictures/${req.file.filename}`
            : undefined;

        const newUser: IUser = new User({
            name,
            surname,
            socialHandle,
            email,
            profilePicture,
            travelId,
            followers,
            following,
            bio,
        });

        const savedUser = await newUser.save();
        //console.log(savedUser);

        res.status(201).json(savedUser);
    } catch (err) {
        res.status(500).json({ message: err instanceof Error ? err.message : 'An error occurred' });
    }
};

export const createKeycloakUser = async (req: Request, res: Response): Promise<void> => {
    try {
        const { sub, socialHandle, name, surname, email } = req.body;

        if (!sub) {
            res.status(400).json({ message: 'User sub (ID) is required' });
        }

        try {
            let user = await User.findOne({ kc_id: sub });

            if (!user) {
                user = new User({
                    kc_id: sub,
                    socialHandle: socialHandle,
                    email: email,
                    name: name,
                    surname: surname,
                    profilePicture: `${process.env.APIURL}/uploads/profilePictures/default.jpg`,
                    travelId: [],
                    followers: [],
                    following: [],
                    friendIds: [],
                });

                await user.save();
                res.status(201).json({ message: 'User synced successfully' });
            } else {
                res.status(200).json({ message: 'User already exists' });
            }
        } catch (error) {
            console.error('Error syncing user:', error);
            res.status(500).json({ message: 'Error syncing user' });
        }
    } catch (err) {
        res.status(500).json({ message: err instanceof Error ? err.message : 'An error occurred' });
    }
}

export const getUser = async (req: Request, res: Response): Promise<void> => {
    try {
        const user = await User.find();
        res.status(200).json(user);
    } catch (err) {
        res.status(500).json({ message: err instanceof Error ? err.message : 'An error occurred' });
    }
}

export const getUserById = async (req: Request, res: Response): Promise<void> => {
    try {
        const user = await User.findOne({kc_id: req.params.userId});
        res.status(200).json(user);
    } catch (err) {
        res.status(500).json({ message: err instanceof Error ? err.message : 'An error occurred' });
    }
}

export const getUserPublicInformationById = async (req: Request, res: Response): Promise<void> => {
    try {
        const user = await User.findOne({kc_id: req.params.userId}, "socialHandle profilePicture");
        res.status(200).json(user);
    } catch (err) {
        res.status(500).json({ message: err instanceof Error ? err.message : 'An error occurred' });
    }
}

export const updateUser = async (req: Request, res: Response): Promise<void> => {
    try {
        const updatedUser = await User.findOneAndUpdate({kc_id: req.params.userId}, req.body, { new: true });
        res.status(200).json(updatedUser);
    } catch (err) {
        res.status(500).json({ message: err instanceof Error ? err.message : 'An error occurred' });
    }
}

export const addProfilePicture = async (req: Request, res: Response): Promise<void> => {
    try {
        const user = await User.findOne({kc_id: req.params.userId});
        const oldProfilePicture = user?.profilePicture?.split('/profilePictures/')[1];
        if (oldProfilePicture && oldProfilePicture !== 'default.jpg') {
            fs.unlink(`uploads/profilePictures/${oldProfilePicture}`, (err) => {
                if (err) {
                    console.error('Error deleting old profile picture:', err);
                }
            });
        }
        const updatedUser = await User.findOneAndUpdate({kc_id: req.params.userId}, { profilePicture: req.file ? `${process.env.APIURL}/uploads/profilePictures/${req.file.filename}` : undefined });
        res.status(200).json(updatedUser);
    } catch (err) {
        res.status(500).json({ message: err instanceof Error ? err.message : 'An error occurred' });
    }
}


export async function searchUserByUsernamechunk(req: Request, res: Response){
    const unameChunk: string = req.params.unameChunk;

    if (!unameChunk || unameChunk.length < 1) {
        return res.status(201).json({error: "Es muss ein Teil eines Nutzernamen angegeben werden"});
    }

    const results = await User.find({ socialHandle: { $regex: unameChunk, $options: 'i' } });

    return res.status(200).json(results);

}

interface KeycloakToken {
    sub: string;
    exp: number;
    iat: number;
}

export async function getFollowers(req: Request, res: Response) {
    try {
        if (!req.headers.authorization || !req.headers.authorization.startsWith('Bearer ')) {
            res.status(401).json({ message: "Not authenticated" });
            return;
        }

        const token = req.headers.authorization.split(' ')[1];
        const sub = await getSubFromToken(token);

        const user = await User.findOne({ kc_id: sub});
        if (!user) {
            res.status(404).json({ message: "User not found" });
            return;
        }
        const followers = await User.find({ kc_id: { $in: user.following } }, "socialHandle profilePicture name surname kc_id");

        return res.status(200).json(followers);


    } catch (err) {
        res.status(500).json({ message: err instanceof Error ? err.message : "An internal server error occurred" });
    }
}

export async function getFollowing(req: Request, res: Response) {
    try {
        if (!req.headers.authorization || !req.headers.authorization.startsWith('Bearer ')) {
            res.status(401).json({message: "Not authenticated"});
            return;
        }

        const token = req.headers.authorization.split(' ')[1];
        const sub = await getSubFromToken(token);
        console.log(sub)

        const user = await User.findOne({kc_id: sub});
        console.log(user?.following);
        if (!user) {
            res.status(404).json({message: "User not found"});
            return;
        }
        const following = await User.find({kc_id: {$in: user.following}}, "socialHandle profilePicture name surname kc_id");

        return res.status(200).json(following);
    } catch (err) {
        res.status(500).json({message: err instanceof Error ? err.message : "An internal server error occurred"});
    }
}


async function getSubFromToken(token: string): Promise<string> {
    let decodedToken: KeycloakToken;
    try {
        decodedToken = jwtDecode<KeycloakToken>(token);
    } catch (err) {
        throw new Error("Invalid token");
    }

    return decodedToken.sub;
}